import json
import logging
from functools import wraps
from typing import Any, Dict, Optional

from sentry_sdk import capture_exception

from filum_utils.sentry import SentryClient



class ErrorType:
    INTERNAL = "internal"
    EXTERNAL = "external"

class ErrorMessage:
    IMPLEMENTATION_ERROR = "Implementation error"
    MISMATCH_LAST_CURRENT_INDEX = "Mismatch last current index"
    MISSING_SEGMENT_ID = "Missing segment id"
    MISSING_FILE = "Missing file"
    INVALID_DATETIME_STRING = "Invalid datetime string format - unable to parse"
    TIMEZONE_ERROR = "Timezone error"


def capture_exceptions_with_sentry(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        SentryClient().setup()
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logging.exception(e)
            # Capture the exception in Sentry
            capture_exception(e)
            return json.dumps({})

    return wrapper


class BaseError(Exception):
    def __init__(
        self,
        message: str,
        data: Optional[Dict[str, Any]] = None,
        error_code: Optional[int] = None,
        error_type: str = ErrorType.INTERNAL
    ):
        self.message = message
        self.data = data
        self.error_code = error_code
        self.error_type = error_type
